// PROJECT  :EEPROMWrite  (with Serial Echo for confirmation)
// PURPOSE  :Support for Nano Coding Companion EEPROM Exercise
//          :Stores word (uint16_t) data pattern to EEPROM to be subsequently read and displayed
//          :on the RSGC ACES Nano Coding Companion device
// COURSE   :ICS3U-E
// AUTHOR   :C. D'Arcy
// DATE     :2022 05 12
// MCU      :328P
// STATUS   :Working!
// REFERENCE:http://darcy.rsgc.on.ca/ACES/TEI3M/ArduinoCode/NanoCodingCompanion.html
// NOTES    :Use your standard Arduino cable; the Sparkfun AVR pocket programmer has issues with EEPROM :(
//          :Byte order: Little Endian
//      
#include <EEPROM.h>
uint16_t pattern [] = { 0B0000000000000000,
                        0B1000000000000001,
                        0B1100000000000011,
                        0B1110000000000111,
                        0B1111000000001111,
                        0B0111100000011110,
                        0B0011110000111100,
                        0B0001111001111000,
                        0B0000111111110000,
                        0B0000011111100000,
                        0B0000001111000000,
                        0B0000011111100000,
                        0B0000111111110000,
                        0B0001111001111000,
                        0B0011110000111100,
                        0B0111100000011110,
                        0B1111000000001111,
                        0B1110000000000111,
                        0B1100000000000011,
                        0B1000000000000001};
uint8_t numElements = sizeof(pattern) / sizeof(uint16_t);
uint16_t addr = 0;
#define SETTLE 20

void setup() {
  Serial.begin(9600);
  while (!Serial);
  Serial.println(numElements);
  EEPROM.write(addr++, lowByte(numElements));
  delay(SETTLE);
  EEPROM.write(addr++, highByte(numElements));
  delay(SETTLE);
  for (uint8_t i = 0; i < numElements; i++) {
    EEPROM.write(addr++, lowByte(pattern[i]));
    delay(SETTLE);
    EEPROM.write(addr++, highByte(pattern[i]));
    delay(SETTLE);
  }
  // echo EEPROM data to the Serial Monitor for confirmation
   for (addr = 2; addr <= numElements<<1; addr+=2){
     Serial.println(readEEPROMWord(addr), HEX);
     delay(SETTLE);
   }
}
/*****************************************************************
 * Reads two EEPROM bytes (Little Endian), assembles a 16-bit word 
 * Precondition: Address within EEPROM range (328P: 1KB)
 * Postcondition: returns 16-bit word of EEPROM data 
 *****************************************************************/
uint16_t readEEPROMWord(uint16_t address) {
  uint8_t low  = EEPROM.read(address++);
  delay(SETTLE);
  uint8_t high = EEPROM.read(address);
  delay(SETTLE);
  return (high << 8) + low;
}

void loop() {}
